#! /usr/bin/env python3

# above is the shebang, which tells machine to use python to run this file
# below is a *module docstring*, telling us what this file is and how to use

"""
This is an improved version of wordcount.py that can be run on the command line.
On the command line, you can pass an argument that points to any text file you
would like to process. So, example use is:

```
> ./count.py <path-to-file.txt>

> python3 count.py <path-to-file.txt
```

It works on any plain text, for example, the following file, which is plaintext
of Lewis Carroll's  Alice's Adventures in Wonderland, found at:
https://www.gutenberg.org/files/11/11-0.txt

If you put this file in your `usr/bin` directory, you can call it from anywhere
on your machine, without writing out the full path to the script each time. You
can name it just `count`, without an extension, and then do:

```
> count <my-file.txt
```

Finally note: you can do `diff -y wordcount.py count.py` to see the difference
between the two scripts.
"""

# we need these two modules in order to run our file as a script (see end)
import os
import sys
# import a tool that counts occurrences of things in python
from collections import Counter


# create a function -- a chunk of code that can be *called* multiple times
def wordcount(filepath):
    """
    This function, called wordcount, takes one argument, `filepath`,
    which is a path to plaintext data.

    It returns a Counter, with word frequencies in this filepath
    """
    # open the file (like double clicking on it)
    with open(filepath) as data:
        raw_text = data.read()
    # print the first 500 characters to screen
    print("Raw data:\n\n", raw_text[:500], "\n\n")
    # make everything lowercase (i.e. normalise)
    lowered = raw_text.lower()
    # split text on spaces. is this reliable?
    wordlist = lowered.split(' ')
    # loop over each word, remove those that are not alphanumeric
    cleaned = [word for word in wordlist if word.strip().isalnum()]
    # perform the counting using the Counter class
    counted = Counter(cleaned)
    # print top 10 most frequent
    print("Counts:\n\n", counted.most_common(10), "\n\n")
    # return the wordcount data to us after we call this function
    return counted


def process(text_file):
    # all we have done so far is *define* a function ...
    # we haven't yet *called* it (i.e. run it). do it like this:
    wordcounts = wordcount(text_file)
    # store the results of the analysis in results.txt
    with open("results.txt", "w") as results_file:
        # loop over the words starting with most frequent
        # or, we could use sorted(wordcounts.most_common()) to alphabetise
        for word, count in wordcounts.most_common():
            # write the word and count to file, separated by a tab
            results_file.write(f"{word}\t{count}\n")
    # also return the counter from this function
    return wordcounts


# this tells python what to do if we call this file as a script
if __name__ == "__main__":
    # get the *argument* we passed to the script (-1 is the last item in a list)
    input_file = sys.argv[-1]
    # check that it exists, raise error if it does not
    if not os.path.isfile(input_file):
        raise ValueError(f"File {input_file} does not exist! Stopping.")
    # if there is no error, we can do the processing
    print(f"Processing {input_file} ...")
    # call process(), which calls wordcount()
    results = process(input_file)
    # the hard part is done. now we just want to tell the user that it worked
    # first, calculate number of items in the counter (i.e. its length)
    uniq = len(results)
    # now, for extra credit: figure out how to calculate the total
    # number of words in our counter object. hint: you may have to loop
    # over the *values* of the Counter object with `results.values()`
    total_words = "???"
    # print our success message
    print(f"All done! {uniq} unique words counted. Total words: {total_words}")
